# coding: utf-8
import pygame
from random import randint

# všimněte si, že třída níže volá přímo funkci pygame.image.load
# (o něco obyčejnější by bylo volat ji v konstruktoru, ale chceme šetřit čas)
# proto je potřeba zavolat pygame.init() před samotnou definicí té třídy
pygame.init()
screenrect = pygame.Rect(0, 0, 640, 480)
screen = pygame.display.set_mode(screenrect.size)
pygame.display.set_caption('Poníci')

black = pygame.Color(0, 0, 0)
white = pygame.Color(255, 255, 255)

class Pony(pygame.sprite.Sprite):
    # rychlost pohybu v pixelech za sekundu
    speed = 50
    # všechny obrázky poníků, co budou potřeba, načteme jednou během spouštění skriptu
    # obrázek PNG s patřičným názvem musíte stáhnout a uložit do stejné složky jako tento skript
    spritesheet = pygame.image.load('spritesheet.png').convert_alpha()
    sprite_size = 32 # velikost jednoho čtverečku ve spritesheetu, v pixelech
    
    def __init__(self, maze):
        """maze: pygame.Mask; určuje, kam poník smí a kam ne"""
        pygame.sprite.Sprite.__init__(self)
        # barva. Číslo v rozsahu 0..7, protože tolik barev máme ve spritesheetu
        self.color = randint(0, 7)
        # směr chůze. 0: dolů, 1: doleva, 2: doprava, 3: nahoru, protože tak je to ve spritesheetu
        self.direction = 0
        # poloha na mapě
        self.border_right = maze.get_size()[0] - self.sprite_size
        self.border_bottom = maze.get_size()[1] - self.sprite_size
        self.x, self.y = randint(0, self.border_right), randint(0, self.border_bottom)
        self.rect = pygame.Rect(self.x, self.y, self.sprite_size, self.sprite_size)
        # na začátku cíl chůze necháme na správném místě
        self.target = self.rect.x, self.rect.y
        # načteme obrázek ze spritesheetu
        self.update_image()
        # zapamatujeme si bludiště pro pozdější kontroly nárazu
        self.maze = maze
        # do masky zaznamenáme pixely obrázku, co nejsou průhledné
        self.mask = pygame.mask.from_surface(self.image)
    
    def update(self, time):
        """Posune poníka směrem k cíli, pokud to jde
        time: float; udává čas od posledního volání"""
        distance = time * self.speed
        # pokaždé zkoušíme, jestli poník chce jít tímhle směrem a jestli může
        if self.target[0] > self.rect.x + distance and self.can_move(distance, 0):
            self.x += distance
            self.direction = 2
        elif self.target[0] < self.rect.x - distance and self.can_move(-distance, 0):
            self.x -= distance
            self.direction = 1
        elif self.target[1] > self.rect.y + distance and self.can_move(0, distance):
            self.y += distance
            self.direction = 0
        elif self.target[1] < self.rect.y - distance and self.can_move(0, -distance):
            self.y -= distance
            self.direction = 3
        # skutečnou polohu překopírujeme do celočíselného self.rect
        self.rect.x, self.rect.y = self.x, self.y
        self.update_image()
    
    def update_image(self):
        """Vybere aktuální obrázek ze spritesheetu, pro animaci"""
        # začneme v levém horním rohu tabulky obsahující poníky příslušné barvy
        cell_x, cell_y = 3 * (self.color % 4), 4 * (self.color // 4)
        # najdeme buňku podle směru chůze a aktuálního času (animace 6 snímků za vteřinu)
        cell_y += self.direction
        time = pygame.time.get_ticks() / 1000.0 # čas od spuštění hry, původně v milisekundách
        cell_x += int(6 * time) % 3
        # přepočítáme pozici v tabulce na skutečné souřadnice v pixelech a zkopírujeme obrázek
        x = cell_x * self.sprite_size
        y = cell_y * self.sprite_size
        self.image = self.spritesheet.subsurface(pygame.Rect((x, y), self.rect.size))
    
    def can_move(self, horz, vert):
        """Ověří, jestli se maska poníka nepřekrývá s maskou překážek
        horz, vert: int; posuv oproti současné pozici poníka"""
        overlap = self.maze.overlap_area(self.mask, (int(self.x + horz), int(self.y + vert)))
        return overlap == 0
    
    def set_target(self, x, y):
        """Změní cíl chůze na dané souřadnice"""
        self.target = [x, y]

def draw_map(image, filename):
    """Vykreslí na daný obrázek černě mapu z textového souboru daného názvu"""
    f = open(filename, "r")
    for line in f.readlines():
        # každý řádek souboru rozdělíme na čtyři hodnoty a ty převedeme na čísla
        x1, y1, x2, y2 = line.split()
        pygame.draw.line(image, black, (int(x1), int(y1)), (int(x2), int(y2)))
    f.close()

background = pygame.Surface(screenrect.size)
background.fill(white)
draw_map(background, "map.txt")
screen.blit(background, (0,0))
pygame.display.update()

# vyrobíme z vykreslené mapy masku: překážka je všechno, co není bílé
# (set_colorkey je malý úskok, kterým nastavíme dočasně bílou jako průhlednost)
background.set_colorkey(white)
maze = pygame.mask.from_surface(background)
background.set_colorkey(None)

ponies = pygame.sprite.RenderUpdates()
peaceful = pygame.sprite.Group()
for i in range(10):
    pony = Pony(maze)
    pony.add(ponies)
    pony.add(peaceful)
killer = pony
peaceful.remove(killer)

clock = pygame.time.Clock()

alive = True
while alive:
    for event in pygame.event.get():
        if event.type == pygame.QUIT or (event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE):
            alive = False
        
        elif event.type == pygame.MOUSEBUTTONDOWN:
            for pony in ponies:
                pony.set_target(event.pos[0], event.pos[1])
    
    ponies.clear(screen, background)
    ponies.update(clock.get_time() / 1000.0)
    for pony in pygame.sprite.spritecollide(killer, peaceful, dokill=False, collided=pygame.sprite.collide_mask):
		pony.kill() # poníci ve skutečnosti neumřou, jen je odebereme ze všech skupin
    dirty = ponies.draw(screen)
    pygame.display.update(dirty)
    
    clock.tick(30) # 30 FPS

pygame.quit()
